﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SfDataGridBugDemo.Droid;
using Syncfusion.Data;
using Syncfusion.SfDataGrid.XForms;
using Xamarin.Forms;

namespace SfDataGridBugDemo
{
    public partial class MainPage : ContentPage
    {
        private MainViewModel Context { get; set; }

        public MainPage()
        {
            InitializeComponent();
        }

        protected override async void OnBindingContextChanged()
        {
            Context = (MainViewModel)BindingContext;
            InitializeSongs();
            await RefreshPlaylistComboBox();
        }

        private void InitializeSongs()
        {
            Context.Songs = new SongList();
        }

        private async Task RefreshPlaylistComboBox()
        {
            await Task.Delay(100);
            PlaylistComboBox.SelectedIndex = 0;
        }

        private void SongsListView_GridViewCreated(object sender, GridViewCreatedEventArgs e)
        {
            InitializeDataGrid();
        }

        private async void PlaylistComboBox_OnSelectedIndexChanged(object sender, EventArgs e)
        {
            await Context.Songs.UpdatePlaylist();
        }


        private void InitializeDataGrid()
        {
            SongsListView.GridStyle = new GridStyle(Color.RoyalBlue);
            SongsListView.Columns[2].HeaderTemplate = CreateHeaderDataTemplate("Title");
            SongsListView.Columns[3].HeaderTemplate = CreateHeaderDataTemplate("Artist");
            SongsListView.Columns[4].HeaderTemplate = CreateHeaderDataTemplate("Album");
            SongsListView.Columns[5].HeaderTemplate = CreateHeaderDataTemplate("Bitrate");
            SongsListView.Columns[6].HeaderTemplate = CreateHeaderDataTemplate("Date");
            SongsListView.Columns[7].HeaderTemplate = CreateHeaderDataTemplate("Path");

            ((GridTemplateColumn)SongsListView.Columns[2]).CellTemplate = CreateCellDataTemplate(nameof(Song.Title), TextAlignment.Start);
            ((GridTemplateColumn)SongsListView.Columns[3]).CellTemplate = CreateCellDataTemplate(nameof(Song.Artist), TextAlignment.Start);
            ((GridTemplateColumn)SongsListView.Columns[4]).CellTemplate = CreateCellDataTemplate(nameof(Song.Album), TextAlignment.Start);
            ((GridTemplateColumn)SongsListView.Columns[5]).CellTemplate = CreateCellDataTemplate(nameof(Song.Bitrate), TextAlignment.Center);
            ((GridTemplateColumn)SongsListView.Columns[6]).CellTemplate = CreateCellDataTemplate(nameof(Song.Date), TextAlignment.Center );
            ((GridTemplateColumn)SongsListView.Columns[7]).CellTemplate = CreateCellDataTemplate(nameof(Song.Path), TextAlignment.Start);

            if (SongsListView.SortColumnDescriptions.Count == 0)
            {
                SongsListView.SortColumnDescriptions.Add(new SortColumnDescription()
                {
                    ColumnName = nameof(Song.MappedPlayed),
                    SortDirection = ListSortDirection.Ascending
                });
            }

            SongsListView.SetBinding(SfDataGrid.ItemsSourceProperty, "Songs.Songs");
        }

        private DataTemplate CreateHeaderDataTemplate(string text)
        {
            return new DataTemplate(() =>
            {
                var grid = new Grid()
                {
                    Padding = new Thickness(10, 5, 0, 5),
                    ColumnDefinitions = new ColumnDefinitionCollection()
                    {
                        new ColumnDefinition(){Width = GridLength.Auto},
                        new ColumnDefinition(){Width = GridLength.Star},
                    }
                };
                var label = new Label() { Text = text, FontSize = 16, LineBreakMode = LineBreakMode.TailTruncation, FontAttributes = FontAttributes.Bold, TextColor = Color.White, HorizontalOptions = LayoutOptions.Center, VerticalOptions = LayoutOptions.Center };
                Grid.SetColumn(label, 1);
                grid.Children.Add(label);

                grid.BackgroundColor = Color.Transparent;

                return grid;
            });
        }

        private DataTemplate CreateCellDataTemplate(string propertyName, TextAlignment align, IValueConverter converter = null)
        {
            return new DataTemplate(() =>
            {
                var label = new Label() { FontSize = 14, LineBreakMode = LineBreakMode.TailTruncation, Margin = new Thickness(10, 0, 0, 0), TextColor = Color.Black, HorizontalTextAlignment = align, VerticalTextAlignment = TextAlignment.Center, HorizontalOptions = LayoutOptions.Center, VerticalOptions = LayoutOptions.Center };
                label.SetBinding(Label.TextProperty, propertyName, BindingMode.OneWay, converter);
                return label;
            });
        }
    }
}
