﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using Android.App;
using Android.Content;
using Android.OS;
using Android.Runtime;
using Android.Views;
using Android.Widget;
using Xamarin.Forms;

namespace SfDataGridBugDemo
{
    public class Song : INotifyPropertyChanged
    {

        public string Title { get; set; }

        public string Artist { get; set; }

        public string Album { get; set; }

        public int Bitrate { get; set; }

        public string Token { get; set; }

        public string Path { get; set; }

        public DateTime Date { get; set; }

        public bool Played { get; set; }

        public Song MappedTitle => this;
        public Song MappedArtist => this;
        public Song MappedBitrate => this;
        public Song MappedAlbum => this;
        public Song MappedPath => this;

        public PlayedBool MappedPlayed => Played ? PlayedBool.Played : PlayedBool.Unplayed;
        public event PropertyChangedEventHandler PropertyChanged;

        public Song(string title, string artist, TimeSpan duration, string album, int bitrate, string token, string path, string unknown, DateTime added)
        {
            Set(title, artist, duration, album, bitrate, token, path, unknown);
        }

        public void Set(string title, string artist, TimeSpan duration, string album, int bitrate, string token, string path, string unknown)
        {
            string oldAlbum = Album;
            string oldTitle = Title;
            string oldArtist = Artist;
            int oldBitrate = Bitrate;
            Title = title;
            Artist = String.IsNullOrEmpty(artist) ? unknown : artist;
            Album = String.IsNullOrEmpty(album) ? unknown : album;
            Bitrate = bitrate;
            Token = token;
            Path = path;

            if (PropertyChanged != null)
            {
                if (Title != oldTitle)
                {
                     InvokePropertyChanged(nameof(MappedTitle));
                }
                if (Artist != oldArtist)
                {
                     InvokePropertyChanged(nameof(MappedArtist));
                }
                if (Album != oldAlbum)
                {
                     InvokePropertyChanged(nameof(MappedAlbum));
                }
                if (Bitrate != oldBitrate)
                {
                     InvokePropertyChanged(nameof(MappedBitrate));
                }
            }
        }

        private void InvokePropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }

    public class PlayedBool : IEquatable<PlayedBool>, IComparable
    {
        public static readonly PlayedBool Played = new PlayedBool(true);
        public static readonly PlayedBool Unplayed = new PlayedBool(false);

        public readonly bool Value;
        public override string ToString()
        {
            return Value ? "   " + "Played songs" : "   " + "Playing songs";
        }

        public int CompareTo(object obj)
        {
            if (obj is PlayedBool pb)
            {
                return Value.CompareTo(pb.Value);
            }
            return 0;
        }

        public PlayedBool(bool played)
        {
            Value = played;
        }

        public bool Equals(PlayedBool other)
        {
            return other != null && Value == other.Value;
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj))
                return false;
            return obj is PlayedBool b && Equals(b);
        }

        public override int GetHashCode()
        {
            return Value.GetHashCode();
        }

    }
}